package server

import (
	"crypto/rand"
	"encoding/hex"
	"fmt"
	"html/template"
	"net/http"
	"strconv"
	"time"

	"github.com/google/uuid"
	"streifling.com/jason/sicherheitsunterweisung/packages/db"
	"streifling.com/jason/sicherheitsunterweisung/packages/types"
)

func DisplayTable(db *db.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		bs, err := db.GetAllOverviewTableData()
		if err != nil {
			http.Error(w, "DisplayTable: *DB.GetAllOverviewTableData(): "+fmt.Sprint(err), http.StatusInternalServerError)
		}
		template.Must(template.ParseFiles("templates/table.html")).ExecuteTemplate(w, "content", bs)
	}
}

func DisplaySearchResults(db *db.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		bs, err := db.GetOverviewTableDataByName(r.PostFormValue("search"))
		if err != nil {
			http.Error(w, "DisplayResults: db.ReadByName(r.PostFormValue()): "+fmt.Sprint(err), http.StatusInternalServerError)
		}
		template.Must(template.ParseFiles("templates/table.html")).ExecuteTemplate(w, "rows", bs)
	}
}

func DisplayInstructorForm(cs chan<- *types.Session) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		session := new(types.Session)
		session.ID = uuid.New()
		cs <- session

		template.Must(template.ParseFiles("templates/briefing.html")).ExecuteTemplate(w, "content", session.ID)
	}
}

func generateUUID() (string, error) {
	bs := make([]byte, 4)

	if _, err := rand.Read(bs); err != nil {
		return "", fmt.Errorf("GenerateUUID: rand.Read(bs): %v\n", err)
	}

	return hex.EncodeToString(bs), nil
}

func AddParticipant(s *types.Session) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		login, err := generateUUID()
		if err != nil {
			http.Error(w, "AddParticipant: generateUUID(): "+fmt.Sprint(err), http.StatusInternalServerError)
		}

		s.Logins = append(s.Logins, login)
		template.Must(template.ParseFiles("templates/briefing.html")).ExecuteTemplate(w, "new", login)
	}
}

// TODO: Hier weiter machen, irgendwie die b.ID herausgeben,
// am besten hier auch die p.IDs rausgeben, damit diese später verknüpft werden können
func SubmitBriefingForm(s *types.Session, db *db.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		now := time.Now()
		briefing := new(types.Briefing)

		// TODO: Dropdownmenü
		// instructorFirstName := r.PostFormValue("instructor-first")
		// instructorLastName := r.PostFormValue("instructor-last")

		briefing.Date = now.Format("2006-01-02")
		briefing.Time = now.Format("15:04:05")
		briefing.Location = r.PostFormValue("location")
		briefing.DocumentName = r.PostFormValue("document") // TODO: in HTML einfügen
		briefing.AsOf = r.PostFormValue("state")            // TODO: Umbenennen
		// briefing.InstructorID = r.PostFormValue("instructor-id") // TODO: aus Dropdown holen

		db.WriteBriefing(briefing)
		s.BriefingID = briefing.ID
	}
}

// TODO: Make it only serve one purpose
func loginIsCorrect(l string, ss []*types.Session) bool {
	for _, session := range ss {
		for i, v := range session.Logins {
			if l == v {
				session.Logins = append(session.Logins[:i], session.Logins[i+1:]...)
				return true
			}
		}
	}
	return false
}

func newParticipant(l string) (*types.Participant, error) {
	var err error
	p := new(types.Participant)

	p.ID, err = strconv.ParseInt(l, 10, 64)
	if err != nil {
		return nil, fmt.Errorf("newParticipant: strconv.Atoi(idString): %v\n", err)
	}

	return p, nil
}

func DisplayParticipantForm(ss []*types.Session) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		type data struct {
			SessionID uuid.UUID
			UUID      string
		}

		if loginIsCorrect(r.PostFormValue("login"), ss) {
			data := new(data)
			var err error

			data.UUID, err = generateUUID()
			if err != nil {
				http.Error(w, "DisplayParticipantForm: generateUUID(): "+fmt.Sprint(err), http.StatusInternalServerError)
			}

			template.Must(template.ParseFiles("templates/participant.html")).ExecuteTemplate(w, "content", data)
		} else {
			template.Must(template.ParseFiles("templates/login.html")).ExecuteTemplate(w, "content", nil)
		}
	}
}

// func readAnswer(r *http.Request, p *types.Participant, i int) error {
// 	v, err := strconv.Atoi(r.PostFormValue("answer"))
// 	if err != nil {
// 		return fmt.Errorf("readAnswer: strconv.Atoi(): %v\n", err)
// 	}
//
// 	p.Questions[i].Chosen = v
//
// 	return nil
// }
//
// func DisplayQuestion(i int, p *types.Participant) http.HandlerFunc {
// 	return func(w http.ResponseWriter, r *http.Request) {
// 		if i == 0 {
// 			p.FirstName = r.PostFormValue("participant-first-" + fmt.Sprintf("%d", p.ID))
// 			p.LastName = r.PostFormValue("participant-last-" + fmt.Sprintf("%d", p.ID))
// 			p.Company = r.PostFormValue("participant-company-" + fmt.Sprintf("%d", p.ID))
// 		} else {
// 			if err := readAnswer(r, p, i-1); err != nil {
// 				http.Error(w, "DisplayQuestion: readAnswer(r, p, i): "+fmt.Sprint(err), http.StatusInternalServerError)
// 			}
// 		}
//
// 		data := new(questionData)
// 		data.ID = p.ID
// 		data.Q = p.Questions[i]
// 		data.I = i
// 		data.J = i + 1
//
// 		template.Must(template.ParseFiles("templates/question.html")).ExecuteTemplate(w, "content", data)
// 	}
// }
//
// func DisplayTestResults(b *types.Briefing, p *types.Participant) http.HandlerFunc {
// 	return func(w http.ResponseWriter, r *http.Request) {
// 		numQuestions := len(p.Questions)
// 		wrongAnswers := make([]int, 0)
// 		fmt.Println(wrongAnswers)
//
// 		if err := readAnswer(r, p, numQuestions-1); err != nil {
// 			http.Error(w, "DisplayTestResults: readAnswer(r, p, i): "+fmt.Sprint(err), http.StatusInternalServerError)
// 		}
//
// 		for i, q := range p.Questions {
// 			if q.Chosen != q.Correct {
// 				wrongAnswers = append(wrongAnswers, i)
// 			}
// 		}
//
// 		if wrongAnswers == nil {
// 			b.Participants = append(b.Participants, p)
// 		} else {
// 			data := new(questionData)
// 			data.ID = p.ID
// 			data.Q = p.Questions[0]
// 			data.I = 0
// 			data.J = data.I + 1
// 			template.Must(template.ParseFiles("templates/question.html")).ExecuteTemplate(w, "content", data)
// 		}
//
// 		template.Must(template.ParseFiles("templates/results.html")).ExecuteTemplate(w, "content", nil)
// 	}
// }
